﻿using System;
using System.Text;
using SpectrumAnalysis.Data;
using SpectrumAnalysis.Interfaces;

namespace SpectrumAnalysis.Calculation
{
    public class SpectrumCalculator : ISpectrumCalculator
    {
        private readonly InterpolationType _interpolation;
        public SpectrumCalculator(InterpolationType interpolation)
        {
            _interpolation = interpolation;
        }

        

            
  
       

       

        public virtual Xyz CalculateXyzFromSpectrum(Spectrum spectrum, StandardObservers observer, 
            StandardIlluminants illuminant)
        {
            


            var observerObj = new StandardObserver(observer);
            var theIlluminant = new StandardIlluminant(illuminant);
            var n = 0.0;
            var x = 0.0;
            var y = 0.0;
            var z = 0.0;
            foreach (var t in spectrum)
            {

                Xyz obsXyz = observerObj.GetXyzForWavelength(t.Wavelength);

                
                
                var illum = theIlluminant.GetIlluminationForWavelength(t.Wavelength);
                var xbar = obsXyz.X;
                var ybar = obsXyz.Y;
                var zbar = obsXyz.Z;
                var deltaX = xbar * illum * t.Transmittance;
                var deltaY = ybar * illum * t.Transmittance;
                var deltaZ = zbar * illum * t.Transmittance;

                x += deltaX;
                y += deltaY;
                z += deltaZ;
                n += ybar * illum;



                

                

            }
            var xnorm = x / n;
            var ynorm = y / n;
            var znorm = z / n;




            return new Xyz(xnorm, ynorm, znorm);
        }

       
        public virtual double CalculateDominantWavelength(IObserver observer, Xy whitepoint, Xy point, 
            out string output)
        {

            DomCompWavelength dcw = CalcDomCompWavelength(observer, whitepoint, point, out output, false);
            return dcw.DominantWavelength ?? 0;
        }

        public virtual DomCompWavelength CalcDomCompWavelength(IObserver observer, Xy whitepoint, Xy point,
            out string output, bool includeComplement)
        {

            var sb = new StringBuilder();
            sb.AppendLine("Point: " + point);
            sb.AppendLine("Observer: " + observer);
            sb.AppendLine("White point: " + whitepoint);




            var offsetPoint = point.Shift(whitepoint);
            sb.AppendLine("Offset point: " + offsetPoint);
            var offsetPolarPoint = new PolarCoord(offsetPoint);
            sb.AppendLine("Offset polar point: " + offsetPolarPoint);


            sb.AppendLine("Interpolation = " + _interpolation);
            var chromaticity = new Chromaticity(observer, whitepoint, _interpolation);
            // Do interpolation

            var dominantWavelength =
                chromaticity.GetInterpolatedWavelength(offsetPolarPoint.Theta);
            sb.AppendLine("Dominant WL = " + dominantWavelength);

            double? complementaryWavelength = null;
            if (includeComplement)
            {
                // Rotate the offsetPolarPoint by 180 or pi degrees around the white point
                // to get the complement
                var compPolarPoint = offsetPolarPoint.GetRotated(Math.PI);
                sb.AppendLine("Complementary polar point = " + compPolarPoint);
                complementaryWavelength = chromaticity.GetInterpolatedWavelength(compPolarPoint.Theta);
                sb.AppendLine("Complementary WL = " + complementaryWavelength.Value);
            }




            output = sb.ToString();
            return new DomCompWavelength(dominantWavelength, complementaryWavelength);

        }
    }
}
