﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MathNet.Numerics.Interpolation;
using SpectrumAnalysis.Data;

namespace SpectrumAnalysis.Calculation
{
    public class SpectrumResampler
    {
        private readonly Spectrum _spectrum;
        private readonly double[] _wavelengths;
        private readonly double[] _transmittances;
        private readonly LinearSpline _interpolator;


        public const int MinWavelength = 400;


        public const int MaxWavelength = 700;


        public const int ResampleInterval = 5;

        public SpectrumResampler(Spectrum spectrum)
        {
            _spectrum = spectrum;
            var num = _spectrum.Count();
            _wavelengths = new double[num];
            _transmittances = new double[num];
            for (var i = 0; i < num; ++i)
            {
                var sp = _spectrum.List[i];
                _wavelengths[i] = sp.Wavelength;
                _transmittances[i] = sp.Transmittance;
            }
            _interpolator = LinearSpline.InterpolateSorted(_wavelengths, _transmittances);

            
        }

        //public Spectrum Resample(int fromNm, int toNm, int intervalNm)
        //{
        //    var resampledSpectrum = new Spectrum();
        //    for (var wl = fromNm; wl <= toNm; wl += intervalNm)
        //    {
        //        if (wl >= MinWavelength && wl <= MaxWavelength)
        //        {
        //            var trans = _interpolator.Interpolate((double) wl);
        //            resampledSpectrum.List.Add(new SpectrumPoint(wl, trans));
        //        }
        //    }
        //    return resampledSpectrum;
        //}

        public Spectrum Resample()
        {
            var resampledSpectrum = new Spectrum();
            for (var wl = MinWavelength; wl <= MaxWavelength; wl += ResampleInterval)
            {
                
                var trans = _interpolator.Interpolate((double)wl);
                resampledSpectrum.List.Add(new SpectrumPoint(wl, trans));
               
            }
            return resampledSpectrum;
        }

        public Spectrum OriginalSpectrum { get { return _spectrum; } }

        public double[] OriginalWavelengths { get { return _wavelengths; } }

        public double[] OriginalTransmittances { get { return _transmittances; } } 
    }
}
