﻿using System;

namespace SpectrumAnalysis.Data
{
    public class ChromaPoint
    {
        public ChromaPoint(double wavelength, Xy xy, Xy xyOffset, PolarCoord pc)
        {
            Wavelength = wavelength;
            X = xy.X;
            Y = xy.Y;
            XOffset = xyOffset.X;
            YOffset = xyOffset.Y;
            R = pc.R;
            Theta = pc.Theta;
        }
        public ChromaPoint(double wavelength, double x, double y, double xOffset, double yOffset,
            double r, double theta)
        {
            Wavelength = wavelength;
            X = x;
            Y = y;
            XOffset = xOffset;
            YOffset = yOffset;
            R = r;
            Theta = theta;
        }
        public readonly double Wavelength;
        public readonly double X;
        public readonly double Y;
        public readonly double XOffset;
        public readonly double YOffset;
        public readonly double R;
        public readonly double Theta;

        public Xy GetXyOffset()
        {
            return new Xy(XOffset, YOffset);
        }

        public Xy GetXy()
        {
            return new Xy(X, Y);
        }

        public PolarCoord GetPolarCoord()
        {
            return new PolarCoord(R, Theta);
        }

        public override string ToString()
        {
            return String.Format("Wavelength={0}, xy={1}, offset={2}, polar={3}",
                Wavelength, GetXy(), GetXyOffset(), GetPolarCoord());
        }
    }
}
