﻿using System;
using SpectrumAnalysis.Utility;

namespace SpectrumAnalysis.Data
{
    /// <summary>
    /// Immutable Polor plane coordinate classd (theta and R)
    /// </summary>
    public class PolarCoord
    {
        public PolarCoord(double radius, double angle)
        {
            R = radius;
            Theta = angle;
        }
        public PolarCoord(Xy xy)
        {
            R = GetR(xy);
            Theta = GetTheta(xy);
        }

        public readonly double R;
        public readonly double Theta;

        private static double GetR(Xy xy)
        {
            var x = xy.X;
            var y = xy.Y;
            return Math.Sqrt(x*x+y*y);
        }

        private static double GetTheta(Xy xy)
        {
            var x = xy.X;
            var y = xy.Y;
            return Math.Atan2(x, y);
        }

        public double GetThetaDegrees()
        {
            return (Theta * 180) / Math.PI;
        }

        /// <summary>
        /// Gets a new PolorCoord rotated by the given angle in radians.
        /// This object is immutable, so it does not change.
        /// </summary>
        /// <param name="rotationRadians"></param>
        /// <returns></returns>
        public PolarCoord GetRotated(double rotationRadians)
        {
            var addTheta = Theta + rotationRadians;

            // Normalize so that the angle is between -pi and pi
            const double TWO_PI = Math.PI*2.0;
            var rotatedAngle = addTheta - TWO_PI*Math.Floor((addTheta + Math.PI)/TWO_PI);
            return new PolarCoord(R, rotatedAngle);
        }

        public override string ToString()
        {
            return string.Format("R = {0}, Theta(rad) = {1}, Theta(deg) = {2}", Util.Round(R,6), 
                Util.Round(Theta,6), Util.Round4(GetThetaDegrees()));
        }
    }
}
