﻿using System;
using System.Collections;
using System.Collections.Generic;
using SpectrumAnalysis.Utility;

namespace SpectrumAnalysis.Data
{
    public class Xy : IEnumerable<double>
    {
        public Xy(IList<double> vect)
        {
            X = vect[0];
            Y = vect[1];
        }
        public Xy(double x, double y)
        {
            X = x;
            Y = y;
       }

        public Xy(PolarCoord polar)
        {
            var r = polar.R;
            var t = polar.Theta;
            X = r*Math.Cos(t);
            Y = r*Math.Sin(t);
        }

        public readonly double X;
        public readonly double Y;


        public IEnumerator<double> GetEnumerator()
        {
            var list = new List<double> {X, Y};
            return list.GetEnumerator();
        }

        public override string ToString()
        {
            return string.Format("[{0}, {1}]", Util.Round(X,6), Util.Round(Y,6));
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public Xy Shift(Xy deltaXy)
        {
            var deltaX = deltaXy.X;
            var deltaY = deltaXy.Y;
            return new Xy(X - deltaX, Y - deltaY);
        }

        public Xy Unshift(Xy deltaXy)
        {
            var deltaX = deltaXy.X;
            var deltaY = deltaXy.Y;
            return new Xy(X + deltaX, Y + deltaY);
        }

        
    }
}
