﻿using SpectrumAnalysis.Calculation;
using SpectrumAnalysis.Data;

namespace SpectrumAnalysis.Interfaces
{
    /// <summary>
    /// Interface to perform spectrum calculations
    /// </summary>
    public interface ISpectrumCalculator
    {
        /// <summary>
        /// Calculate XYZ point from a transmittance spectrum
        /// </summary>
        /// <param name="spectrum">Input spectrum</param>
        /// <param name="observer">Observer in StandardObservers enumeration</param>
        /// <param name="illuminant">Illuminant in StandardIlluminants enumeration</param>
        /// <returns></returns>
        Xyz CalculateXyzFromSpectrum(Spectrum spectrum, StandardObservers observer,
            StandardIlluminants illuminant);

        /// <summary>
        /// Calculate Dominant wavelength for an xy(Y) point
        /// </summary>
        /// <param name="whitepoint"></param>
        /// <param name="point">The xy(Y) point to calculate Dom WL for</param>
        /// <param name="output">Output comments on calculation</param>
        /// <param name="observer"></param>
        /// <returns></returns>
        double CalculateDominantWavelength(IObserver observer, Xy whitepoint, Xy point, out string output);

        DomCompWavelength CalcDomCompWavelength(IObserver observer, Xy whitepoint, Xy point, out string output,
            bool includeComplement);


        //double CalculateComplementaryWavelength(IChromaticity chromaticity, Xy point);
    }
}
