﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MathNet.Numerics;
using MathNet.Numerics.LinearAlgebra;

namespace SpectrumAnalysis.Utility
{
    public class TrilinearInterpolator
    {
        private readonly double[] _wavelengths;
        private readonly Vector<double>[] _points;

        public TrilinearInterpolator(double[] wavelengths, double[] xPoints, double[] yPoints, double[] zPoints)
        {
            var len = wavelengths.Length;
            _wavelengths = new double[len];
            _points = new Vector<double>[len];
            for (var i = 0; i < len; ++i)
            {
                _wavelengths[i] = wavelengths[i];
                _points[i] = Vector<double>.Build.DenseOfArray(new[] {xPoints[i], yPoints[i], zPoints[i]});
            }
        }

        public Vector<double> Interpolate(double wavelength)
        {
            Vector<double> point0 = null;
            int i0 = -1;
            int i1 = -1;
            int i2 = -1;
            for (var i = 0; i < _wavelengths.Length - 1; ++i)
            {
                
                if (_wavelengths[i] == wavelength)
                {
                    i0 = i;
                    break;
                }
                else if (_wavelengths[i] < wavelength && _wavelengths[i+1] > wavelength)
                {
                    i1 = i;
                    i2 = i + 1;
                    break;
                }

            }

            if (i0 < 0 && i1 < 0 && i2 < 0)
            {
                throw new Exception("XYZ data not found for wavelength " + wavelength);
            }

            if (i0>=0)
            {
                // The wavelength exactly matches a data point
                return _points[i0];
            }
            
            // Distance between the two data vectors
            Vector<double> point1 = _points[i1];
            Vector<double> point2 = _points[i2];
            

            double wavelength1 = _wavelengths[i1];
            double wavelength2 = _wavelengths[i2];
            double wavelengthOffset = wavelength - wavelength1;
            double wavelengthInterval = wavelength2 - wavelength1;
            double fractionOffset = wavelengthOffset / wavelengthInterval;



            Vector<double> vectorDiff = point2 - point1;
            Vector<double> vectorOffset = vectorDiff.Multiply(fractionOffset);
            Vector<double> interpolatedVector = point1 + vectorOffset;

            return interpolatedVector;

        }
    }
}
