﻿using System;
using System.IO;
using System.Reflection;
using SpectrumAnalysis.Data;


namespace SpectrumAnalysis.Wpf
{
    public class AppSettings
    {
        public static String SpectrumLoadDirectory
        {
            get
            {
                if (String.IsNullOrWhiteSpace(DefaultSettings.SpectrumLoadDirectory))
                {
                    
                    var executableDir = ExecutableDirectory;
                    var dir = executableDir;
                    var testFileDir = Path.Combine(executableDir, "TestFiles");
                    if (Directory.Exists(testFileDir))
                    {
                        dir = testFileDir;
                    }
                    DefaultSettings.SpectrumLoadDirectory = dir;
                    DefaultSettings.Save();
                }
                return DefaultSettings.SpectrumLoadDirectory;
            }
            set
            {
                if (!String.IsNullOrWhiteSpace(value))
                {
                    DefaultSettings.SpectrumLoadDirectory = value;
                    DefaultSettings.Save();
                }
            }
        }

        public static String InfoSaveDirectory
        {
            get
            {
                if (String.IsNullOrWhiteSpace(DefaultSettings.InfoSaveDirectory))
                {
                    DefaultSettings.InfoSaveDirectory = GetMyDocumentsDirectory;
                    DefaultSettings.Save();
                }
                return DefaultSettings.InfoSaveDirectory;
            }
            set
            {
                if (!String.IsNullOrWhiteSpace(value))
                {
                    DefaultSettings.InfoSaveDirectory = value;
                    DefaultSettings.Save();
                }
            }
        }

        public static String GetMyDocumentsDirectory
        {
            get
            {
                return Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            }
        }
        public static String ExecutableDirectory
        {
            get
            {
                var executablePath = Assembly.GetExecutingAssembly().Location;
                var pathArr = executablePath.Split('\\');
                var dirArr = new String[pathArr.Length - 1];
                Array.Copy(pathArr, dirArr, pathArr.Length - 1);
                var executableDir = String.Join("\\", dirArr);
                return executableDir;
            }
        }

        public static String ResourcesDirectory
        {
            get { return Path.Combine(ExecutableDirectory, "Resources"); }
        }

        public static SpectrumType LoadedSpectrumType
        {
            get
            {
                if (String.IsNullOrWhiteSpace(DefaultSettings.LoadedSpectrumType))
                {
                    DefaultSettings.LoadedSpectrumType = SpectrumType.TRANSMITTANCE.ToString();
                    DefaultSettings.Save();
                }
                
                return (SpectrumType)Enum.Parse(typeof(SpectrumType), DefaultSettings.LoadedSpectrumType);
            }
            set
            {
                DefaultSettings.LoadedSpectrumType = value.ToString();
                DefaultSettings.Save();
            }
        }

        private static Properties.Settings DefaultSettings
        {
            get { return Properties.Settings.Default;  }
        }
        
    }
}
